"""
__version__.py
~~~~~~~~~~~~~~

Information about the current version of the colormap-2d package.
"""

__title__ = 'pycolormap-2d'
__description__ = 'A Python package mapping 2D coordinates to colors based ' \
                  'on different 2D color maps. '
__version__ = '1.1.6'
__author__ = 'Thilo Spinner'
__author_email__ = 'thilo.spinner@uni-konstanz.de'
__license__ = 'Apache 2.0'
__url__ = 'https://github.com/spinthil/pycolormap-2d'
